#ifndef TETRAKAIDECAHEDRA_CPP
#define TETRAKAIDECAHEDRA_CPP

// grain deformation class developed by Rongshan Qin at GIFT, POSTECH
// the paper is being published at ISIJ International

#include "Tetrakaidecahedra.h"

Tetrakaidecahedra::Tetrakaidecahedra(int numberOfGrainX, int numberOfGrainY, int numberOfGrainZ, double grainSize)
:ngx(numberOfGrainX), ngy(numberOfGrainY), ngz(numberOfGrainZ), gs(grainSize)
{
	if(gs < 1.0E-10){
	   printf("grain size is too small, mission aborted\n"); 
	   exit(1);
	}
	hgs = gs/2.0;
	qgs = gs/4.0; 
	ngt = 2*ngx*ngy*ngz;
	//ngt = (((ngx+1)/2)*((ngy+1)/2)*((ngz+1)/2)+(ngx/2)*(ngy/2)*(ngz/2));
	dbacurate = 1.0E-9;
	if(ngt == 0){
	   printf("Not available for ordering zero number of tetrakaidecahedra, mission aborted\n"); 
	   exit(1);
	}
	xtg = ngx+1; ytg = ngy+1; ztg = ngz+1;
	//xtg = (ngx+1)/2+1; ytg = (ngy+1)/2+1; ztg = (ngz+1)/2+1;
	nvt = xtg * ytg * ztg *12;
	
	vcx = new double[nvt];
	vcy = new double[nvt];
	vcz = new double[nvt];
	vtx = new double[72];
	sij = new double[9];
	if(vcx == NULL || vcy == NULL || vcz == NULL){
		printf("low memory, aborted\n"); 
		exit(1);
	}
	rdm.setSeed(4407);
	volfile.open("thisVolumeDistri0505.dat");
}

Tetrakaidecahedra::~Tetrakaidecahedra()
{
	if(vcx != NULL)
		delete [] vcx;
	if(vcy != NULL)
		delete [] vcy;
	if(vcz != NULL)
		delete [] vcz;
	delete [] vtx;
	delete [] sij;
}

void Tetrakaidecahedra::outputVertex(char* filename){
  file.open(filename);
  double xm, ym, zm, tm;
  xm = hgs*(1 + ngx);
  ym = hgs*(1 + ngy);
  zm = hgs*(1 + ngz);
  if(xm>ym)
	  tm=xm;
  else
	  tm=ym;
  if(tm<zm)
	  tm=zm;
  maxdim = (float)tm;
  file<<maxdim<<" "<<ngx<<" "<<ngy<<" "<<ngz<<" "<<nvt<<" "<<gs<<" ";
  for(int i=0; i<nvt; i++)
	  file<<float(vcx[i])<<" "<<float(vcy[i])<<" "<<float(vcz[i])<<" ";
  file.close();
}

void Tetrakaidecahedra::setDeformationMatrix(double s11, double s12, double s13, 
											double s21, double s22, double s23, 
											double s31, double s32, double s33)
{
	sij[0]=s11; sij[1]=s12, sij[2]=s13;
	sij[3]=s21; sij[4]=s22, sij[5]=s23;
	sij[6]=s31; sij[7]=s32, sij[8]=s33;
}

void Tetrakaidecahedra::setPlainDeformMatrix(double s11)
{
	if((float)s11 == 0.0){
		printf("s11=0 is not allowed\n"); exit(0);
	}
	setDeformationMatrix(s11, 0, 0, 0, 1, 0, 0, 0, 1.0/s11);
}

void Tetrakaidecahedra::setAxisymmetricTensionDeformMatrix(double s11)
{
	if((float)s11 == 0.0){
		printf("s11=0 is not allowed\n"); exit(0);
	}
	setDeformationMatrix(s11, 0, 0, 0, 1.0/sqrt(s11), 0, 0, 0, 1.0/sqrt(s11));
}

void Tetrakaidecahedra::setAxisymmetricCompressionDeformMatrix(double s33)
{
	if((float)s33 == 0.0){
		printf("s33=0 is not allowed\n"); exit(0);
	}
	setDeformationMatrix(1.0/sqrt(s33), 0, 0, 0, 1.0/sqrt(s33), 0, 0, 0, s33);
}

void Tetrakaidecahedra::setSimpleShearDeformMatrix(double s13)
{
	setDeformationMatrix(1, 0, s13, 0, 1, 0, 0, 0, 1);
}

void Tetrakaidecahedra::deformVtx()
{
	double temx, temy, temz;
	for(int i=0; i<nvt; i++){
		temx = sij[0] * vcx[i] + sij[1] * vcy[i] + sij[2] * vcz[i];
		temy = sij[3] * vcx[i] + sij[4] * vcy[i] + sij[5] * vcz[i];
		temz = sij[6] * vcx[i] + sij[7] * vcy[i] + sij[8] * vcz[i];
		vcx[i] = temx;
		vcy[i] = temy;
		vcz[i] = temz;
	}
}

void Tetrakaidecahedra::calcVtxCoord()
{
	int vertt = 0; // just a parameter
	double x0, x1, x2, x3, y0, y1,y2,y3, z0, z1, z2, z3; // just parameters
	
	for(int i=0; i<xtg; i++){
		x0 = i*gs; x1=x0+qgs; x2=x1+qgs; x3=x2+qgs;  
		for(int j=0; j<ytg; j++){
			y0 = j*gs; y1=y0+qgs; y2=y1+qgs; y3=y2+qgs;  
			for(int k=0; k<ztg; k++){
				z0 = k*gs; z1=z0+qgs; z2=z1+qgs; z3 =z2+qgs;
				// z plane back
				// 0
				vcx[vertt] = x2; vcy[vertt] = y3; vcz[vertt] = z0; vertt++;
				// 1
				vcx[vertt] = x1; vcy[vertt] = y2; vcz[vertt] = z0; vertt++;
				// 2
				vcx[vertt] = x2; vcy[vertt] = y1; vcz[vertt] = z0; vertt++;
				// 3
				vcx[vertt] = x3; vcy[vertt] = y2; vcz[vertt] = z0; vertt++;

				// y plane bottom
				// 4
				vcx[vertt] = x2; vcy[vertt] = y0; vcz[vertt] = z1; vertt++;
				// 5
				vcx[vertt] = x1; vcy[vertt] = y0; vcz[vertt] = z2; vertt++;
				// 6
				vcx[vertt] = x2; vcy[vertt] = y0; vcz[vertt] = z3; vertt++;
				// 7
				vcx[vertt] = x3; vcy[vertt] = y0; vcz[vertt] = z2; vertt++;

				// x plane left
				// 8
				vcx[vertt] = x0; vcy[vertt] = y3; vcz[vertt] = z2; vertt++;
				// 9
				vcx[vertt] = x0; vcy[vertt] = y2; vcz[vertt] = z3; vertt++;
				// 10
				vcx[vertt] = x0; vcy[vertt] = y1; vcz[vertt] = z2; vertt++;
				// 11
				vcx[vertt] = x0; vcy[vertt] = y2; vcz[vertt] = z1; vertt++;
			}
		}
	}
}

// put it back here
void Tetrakaidecahedra::setOneTetrakaideVtx(int & xpos, int & ypos, int & zpos)
{
	int vertt;
	int xp, yp, zp;
	if(xpos%2==1 && ypos%2==1 && zpos%2==1){
		xp = (xpos-1)/2; yp = (ypos-1)/2; zp = (zpos-1)/2;
		vertt = (((xp+1)*ytg+yp+1)*ztg+zp+1)*12;
		vcx[vertt+5] = vtx[0];
		vcy[vertt+5] = vtx[1];
		vcz[vertt+5] = vtx[2];

		vcx[vertt+10] = vtx[3];
		vcy[vertt+10] = vtx[4];
		vcz[vertt+10] = vtx[5];

		vertt = ((xp*ytg+yp+1)*ztg+zp+1)*12;
		vcx[vertt+7] = vtx[6];
		vcy[vertt+7] = vtx[7];
		vcz[vertt+7] = vtx[8];

		vertt = (((xp+1)*ytg+yp)*ztg+zp+1)*12;
		vcx[vertt+8] = vtx[9];
		vcy[vertt+8] = vtx[10];
		vcz[vertt+8] = vtx[11];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp)*12;
		vcx[vertt+5] = vtx[12];
		vcy[vertt+5] = vtx[13];
		vcz[vertt+5] = vtx[14];

		vertt = (((xp+1)*ytg+yp)*ztg+zp)*12;
		vcx[vertt+8] = vtx[15];
		vcy[vertt+8] = vtx[16];
		vcz[vertt+8] = vtx[17];

		vertt = ((xp*ytg+yp+1)*ztg+zp)*12;
		vcx[vertt+7] = vtx[18];
		vcy[vertt+7] = vtx[19];
		vcz[vertt+7] = vtx[20];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp)*12;
		vcx[vertt+10] = vtx[21];
		vcy[vertt+10] = vtx[22];
		vcz[vertt+10] = vtx[23];

		vertt = ((xp*ytg+yp+1)*ztg+zp+1)*12;
		vcx[vertt+4] = vtx[24];
		vcy[vertt+4] = vtx[25];
		vcz[vertt+4] = vtx[26];

		vertt = ((xp*ytg+yp+1)*ztg+zp+1)*12;
		vcx[vertt+2] = vtx[27];
		vcy[vertt+2] = vtx[28];
		vcz[vertt+2] = vtx[29];

		vertt = ((xp*ytg+yp+1)*ztg+zp)*12;
		vcx[vertt+6] = vtx[30];
		vcy[vertt+6] = vtx[31];
		vcz[vertt+6] = vtx[32];

		vertt = ((xp*ytg+yp)*ztg+zp+1)*12;
		vcx[vertt] = vtx[33];
		vcy[vertt] = vtx[34];
		vcz[vertt] = vtx[35];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp)*12;
		vcx[vertt+6] = vtx[36];
		vcy[vertt+6] = vtx[37];
		vcz[vertt+6] = vtx[38];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp+1)*12;
		vcx[vertt+2] = vtx[39];
		vcy[vertt+2] = vtx[40];
		vcz[vertt+2] = vtx[41];
 
		vertt = (((xp+1)*ytg+yp+1)*ztg+zp+1)*12;
		vcx[vertt+4] = vtx[42];
		vcy[vertt+4] = vtx[43];
		vcz[vertt+4] = vtx[44];

		vertt = (((xp+1)*ytg+yp)*ztg+zp+1)*12;
		vcx[vertt] = vtx[45];
		vcy[vertt] = vtx[46];
		vcz[vertt] = vtx[47];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp+1)*12;
		vcx[vertt+1] = vtx[48];
		vcy[vertt+1] = vtx[49];
		vcz[vertt+1] = vtx[50];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp)*12;
		vcx[vertt+9] = vtx[51];
		vcy[vertt+9] = vtx[52];
		vcz[vertt+9] = vtx[53];
       
		vertt = ((xp*ytg+yp+1)*ztg+zp+1)*12;
		vcx[vertt+3] = vtx[54];
		vcy[vertt+3] = vtx[55];
		vcz[vertt+3] = vtx[56];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp+1)*12;
		vcx[vertt+11] = vtx[57];
		vcy[vertt+11] = vtx[58];
		vcz[vertt+11] = vtx[59];

		vertt = (((xp+1)*ytg+yp)*ztg+zp+1)*12;
		vcx[vertt+1] = vtx[60];
		vcy[vertt+1] = vtx[61];
		vcz[vertt+1] = vtx[62];

		vertt = (((xp+1)*ytg+yp)*ztg+zp+1)*12;
		vcx[vertt+11] = vtx[63];
		vcy[vertt+11] = vtx[64];
		vcz[vertt+11] = vtx[65];

		vertt = ((xp*ytg+yp)*ztg+zp+1)*12;
		vcx[vertt+3] = vtx[66];
		vcy[vertt+3] = vtx[67];
		vcz[vertt+3] = vtx[68];
		
		vertt = (((xp+1)*ytg+yp)*ztg+zp)*12;
		vcx[vertt+9] = vtx[69];
		vcy[vertt+9] = vtx[70];
		vcz[vertt+9] = vtx[71];	
	}else if(xpos%2==0 && ypos%2==0 && zpos%2==0){
		xp = xpos/2; yp = ypos/2; zp = zpos/2;
		vertt = ((xp*ytg+yp)*ztg+zp+1)*12;
	
		vcx[vertt+3] = vtx[0];
		vcy[vertt+3] = vtx[1];
		vcz[vertt+3] = vtx[2];

		vcx[vertt] = vtx[3];
		vcy[vertt] = vtx[4];
		vcz[vertt] = vtx[5];

		vcx[vertt+1] = vtx[6];
		vcy[vertt+1] = vtx[7];
		vcz[vertt+1] = vtx[8];

		vcx[vertt+2] = vtx[9];
		vcy[vertt+2] = vtx[10];
		vcz[vertt+2] = vtx[11];

		vertt = ((xp*ytg+yp)*ztg+zp)*12;
		vcx[vertt+3] = vtx[12];
		vcy[vertt+3] = vtx[13];
		vcz[vertt+3] = vtx[14];

		vcx[vertt+2] = vtx[15];
		vcy[vertt+2] = vtx[16];
		vcz[vertt+2] = vtx[17];

		vcx[vertt+1] = vtx[18];
		vcy[vertt+1] = vtx[19];
		vcz[vertt+1] = vtx[20];

		vcx[vertt+0] = vtx[21];
		vcy[vertt+0] = vtx[22];
		vcz[vertt+0] = vtx[23];

		vcx[vertt+9] = vtx[24];
		vcy[vertt+9] = vtx[25];
		vcz[vertt+9] = vtx[26];

		vcx[vertt+8] = vtx[27];
		vcy[vertt+8] = vtx[28];
		vcz[vertt+8] = vtx[29];

		vcx[vertt+11] = vtx[30];
		vcy[vertt+11] = vtx[31];
		vcz[vertt+11] = vtx[32];

		vcx[vertt+10] = vtx[33];
		vcy[vertt+10] = vtx[34];
		vcz[vertt+10] = vtx[35];

		vertt = (((xp+1)*ytg+yp)*ztg+zp)*12;
		vcx[vertt+11] = vtx[36];
		vcy[vertt+11] = vtx[37];
		vcz[vertt+11] = vtx[38];

		vcx[vertt+8] = vtx[39];
		vcy[vertt+8] = vtx[40];
		vcz[vertt+8] = vtx[41];

		vcx[vertt+9] = vtx[42];
		vcy[vertt+9] = vtx[43];
		vcz[vertt+9] = vtx[44];

		vcx[vertt+10] = vtx[45];
		vcy[vertt+10] = vtx[46];
		vcz[vertt+10] = vtx[47];

		vertt = ((xp*ytg+yp+1)*ztg+zp)*12;
		vcx[vertt+7] = vtx[48];
		vcy[vertt+7] = vtx[49];
		vcz[vertt+7] = vtx[50];

		vcx[vertt+4] = vtx[51];
		vcy[vertt+4] = vtx[52];
		vcz[vertt+4] = vtx[53];

		vcx[vertt+5] = vtx[54];
		vcy[vertt+5] = vtx[55];
		vcz[vertt+5] = vtx[56];

		vcx[vertt+6] = vtx[57];
		vcy[vertt+6] = vtx[58];
		vcz[vertt+6] = vtx[59];

		vertt = ((xp*ytg+yp)*ztg+zp)*12;
		vcx[vertt+7] = vtx[60];
		vcy[vertt+7] = vtx[61];
		vcz[vertt+7] = vtx[62];

		
		vcx[vertt+6] = vtx[63];
		vcy[vertt+6] = vtx[64];
		vcz[vertt+6] = vtx[65];

		
		vcx[vertt+5] = vtx[66];
		vcy[vertt+5] = vtx[67];
		vcz[vertt+5] = vtx[68];

		vcx[vertt+4] = vtx[69];
		vcy[vertt+4] = vtx[70];
		vcz[vertt+4] = vtx[71];
	}else{
		printf("the Tetra that you requested is neither odd nor even, aborted\n"); 
		exit(1);
	}
}

void Tetrakaidecahedra::getOneTetrakaideVtx(int & xpos, int & ypos, int & zpos){
	int vertt;
	int xp, yp, zp;
	if(xpos >= ngx || xpos < 0 || ypos >= ngy || ypos < 0 || zpos >= ngz || zpos < 0 ){
		printf("the Tetra that you requested is out of board, aborted\n"); exit(1);}
	else if(xpos%2==1 && ypos%2==1 && zpos%2==1){
		xp = (xpos-1)/2; yp = (ypos-1)/2; zp = (zpos-1)/2;
		vertt = (((xp+1)*ytg+yp+1)*ztg+zp+1)*12;
		vtx[0] = vcx[vertt+5];
		vtx[1] = vcy[vertt+5];
		vtx[2] = vcz[vertt+5];

		vtx[3] = vcx[vertt+10];
		vtx[4] = vcy[vertt+10];
		vtx[5] = vcz[vertt+10];

		vertt = ((xp*ytg+yp+1)*ztg+zp+1)*12;
		vtx[6] = vcx[vertt+7];
		vtx[7] = vcy[vertt+7];
		vtx[8] = vcz[vertt+7];

		vertt = (((xp+1)*ytg+yp)*ztg+zp+1)*12;
		vtx[9] = vcx[vertt+8];
		vtx[10] = vcy[vertt+8];
		vtx[11] = vcz[vertt+8];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp)*12;
		vtx[12] = vcx[vertt+5];
		vtx[13] = vcy[vertt+5];
		vtx[14] = vcz[vertt+5];

		vertt = (((xp+1)*ytg+yp)*ztg+zp)*12;
		vtx[15] = vcx[vertt+8];
		vtx[16] = vcy[vertt+8];
		vtx[17] = vcz[vertt+8];

		vertt = ((xp*ytg+yp+1)*ztg+zp)*12;
		vtx[18] = vcx[vertt+7];
		vtx[19] = vcy[vertt+7];
		vtx[20] = vcz[vertt+7];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp)*12;
		vtx[21] = vcx[vertt+10];
		vtx[22] = vcy[vertt+10];
		vtx[23] = vcz[vertt+10];

		vertt = ((xp*ytg+yp+1)*ztg+zp+1)*12;
		vtx[24] = vcx[vertt+4];
		vtx[25] = vcy[vertt+4];
		vtx[26] = vcz[vertt+4];

		vertt = ((xp*ytg+yp+1)*ztg+zp+1)*12;
		vtx[27] = vcx[vertt+2];
		vtx[28] = vcy[vertt+2];
		vtx[29] = vcz[vertt+2];

		vertt = ((xp*ytg+yp+1)*ztg+zp)*12;
		vtx[30] = vcx[vertt+6];
		vtx[31] = vcy[vertt+6];
		vtx[32] = vcz[vertt+6];

		vertt = ((xp*ytg+yp)*ztg+zp+1)*12;
		vtx[33] = vcx[vertt];
		vtx[34] = vcy[vertt];
		vtx[35] = vcz[vertt];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp)*12;
		vtx[36] = vcx[vertt+6];
		vtx[37] = vcy[vertt+6];
		vtx[38] = vcz[vertt+6];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp+1)*12;
		vtx[39] = vcx[vertt+2];
		vtx[40] = vcy[vertt+2];
		vtx[41] = vcz[vertt+2];
 
		vertt = (((xp+1)*ytg+yp+1)*ztg+zp+1)*12;
		vtx[42] = vcx[vertt+4];
		vtx[43] = vcy[vertt+4];
		vtx[44] = vcz[vertt+4];

		vertt = (((xp+1)*ytg+yp)*ztg+zp+1)*12;
		vtx[45] = vcx[vertt];
		vtx[46] = vcy[vertt];
		vtx[47] = vcz[vertt];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp+1)*12;
		vtx[48] = vcx[vertt+1];
		vtx[49] = vcy[vertt+1];
		vtx[50] = vcz[vertt+1];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp)*12;
		vtx[51] = vcx[vertt+9];
		vtx[52] = vcy[vertt+9];
		vtx[53] = vcz[vertt+9];
       
		vertt = ((xp*ytg+yp+1)*ztg+zp+1)*12;
		vtx[54] = vcx[vertt+3];
		vtx[55] = vcy[vertt+3];
		vtx[56] = vcz[vertt+3];

		vertt = (((xp+1)*ytg+yp+1)*ztg+zp+1)*12;
		vtx[57] = vcx[vertt+11];
		vtx[58] = vcy[vertt+11];
		vtx[59] = vcz[vertt+11];

		vertt = (((xp+1)*ytg+yp)*ztg+zp+1)*12;
		vtx[60] = vcx[vertt+1];
		vtx[61] = vcy[vertt+1];
		vtx[62] = vcz[vertt+1];

		vertt = (((xp+1)*ytg+yp)*ztg+zp+1)*12;
		vtx[63] = vcx[vertt+11];
		vtx[64] = vcy[vertt+11];
		vtx[65] = vcz[vertt+11];

		vertt = ((xp*ytg+yp)*ztg+zp+1)*12;
		vtx[66] = vcx[vertt+3];
		vtx[67] = vcy[vertt+3];
		vtx[68] = vcz[vertt+3];
		
		vertt = (((xp+1)*ytg+yp)*ztg+zp)*12;
		vtx[69] = vcx[vertt+9];
		vtx[70] = vcy[vertt+9];
		vtx[71] = vcz[vertt+9];	
	}else if(xpos%2==0 && ypos%2==0 && zpos%2==0){
		xp = xpos/2; yp = ypos/2; zp = zpos/2;
		vertt = ((xp*ytg+yp)*ztg+zp+1)*12;	
		vtx[0] = vcx[vertt+3];
		vtx[1] = vcy[vertt+3];
		vtx[2] = vcz[vertt+3];

		vtx[3] = vcx[vertt];
		vtx[4] = vcy[vertt];
		vtx[5] = vcz[vertt];

		vtx[6] = vcx[vertt+1];
		vtx[7] = vcy[vertt+1];
		vtx[8] = vcz[vertt+1];

		vtx[9] = vcx[vertt+2];
		vtx[10] = vcy[vertt+2];
		vtx[11] = vcz[vertt+2];

		vertt = ((xp*ytg+yp)*ztg+zp)*12;
		vtx[12] = vcx[vertt+3];
		vtx[13] = vcy[vertt+3];
		vtx[14] = vcz[vertt+3];

		vtx[15] = vcx[vertt+2];
		vtx[16] = vcy[vertt+2];
		vtx[17] = vcz[vertt+2];

		vtx[18] = vcx[vertt+1];
		vtx[19] = vcy[vertt+1];
		vtx[20] = vcz[vertt+1];

		vtx[21] = vcx[vertt+0];
		vtx[22] = vcy[vertt+0];
		vtx[23] = vcz[vertt+0];

		vtx[24] = vcx[vertt+9];
		vtx[25] = vcy[vertt+9];
		vtx[26] = vcz[vertt+9];

		vtx[27] = vcx[vertt+8];
		vtx[28] = vcy[vertt+8];
		vtx[29] = vcz[vertt+8];

		vtx[30] = vcx[vertt+11];
		vtx[31] = vcy[vertt+11];
		vtx[32] = vcz[vertt+11];

		vtx[33] = vcx[vertt+10];
		vtx[34] = vcy[vertt+10];
		vtx[35] = vcz[vertt+10];

		vertt = (((xp+1)*ytg+yp)*ztg+zp)*12;
		vtx[36] = vcx[vertt+11];
		vtx[37] = vcy[vertt+11];
		vtx[38] = vcz[vertt+11];

		vtx[39] = vcx[vertt+8];
		vtx[40] = vcy[vertt+8];
		vtx[41] = vcz[vertt+8];

		vtx[42] = vcx[vertt+9];
		vtx[43] = vcy[vertt+9];
		vtx[44] = vcz[vertt+9];

		vtx[45] = vcx[vertt+10];
		vtx[46] = vcy[vertt+10];
		vtx[47] = vcz[vertt+10];

		vertt = ((xp*ytg+yp+1)*ztg+zp)*12;
		vtx[48] = vcx[vertt+7];
		vtx[49] = vcy[vertt+7];
		vtx[50] = vcz[vertt+7];

		vtx[51] = vcx[vertt+4];
		vtx[52] = vcy[vertt+4];
		vtx[53] = vcz[vertt+4];

		vtx[54] = vcx[vertt+5];
		vtx[55] = vcy[vertt+5];
		vtx[56] = vcz[vertt+5];

		vtx[57] = vcx[vertt+6];
		vtx[58] = vcy[vertt+6];
		vtx[59] = vcz[vertt+6];

		vertt = ((xp*ytg+yp)*ztg+zp)*12;
		vtx[60] = vcx[vertt+7];
		vtx[61] = vcy[vertt+7];
		vtx[62] = vcz[vertt+7];

		
		vtx[63] = vcx[vertt+6];
		vtx[64] = vcy[vertt+6];
		vtx[65] = vcz[vertt+6];

		vtx[66] = vcx[vertt+5];
		vtx[67] = vcy[vertt+5];
		vtx[68] = vcz[vertt+5];

		vtx[69] = vcx[vertt+4];
		vtx[70] = vcy[vertt+4];
		vtx[71] = vcz[vertt+4];	
	}else{
		printf("the Tetra that you requested is neither odd nor even, aborted\n"); 
		exit(1);
	}
}

double Tetrakaidecahedra::distance(int point1, int point2)
{
	double *pt1=vtx+3*point1;
	double *pt2=vtx+3*point2;
	double dist;
	dist = sqrt((*pt1-*pt2)*(*pt1-*pt2)+(*(pt1+1)-*(pt2+1))*(*(pt1+1)-*(pt2+1))
		+(*(pt1+2)-*(pt2+2))*(*(pt1+2)-*(pt2+2)));		
	pt1=NULL;
	pt2=NULL;
	return dist;
}

double Tetrakaidecahedra::trangleArea(int point1, int point2, int point3)
{
	double a, b, c,s;
	a=distance(point1, point2);
	b=distance(point1, point3);
	c=distance(point2, point3);
	s=0.5*(a+b+c);
	return sqrt(s*(s-a)*(s-b)*(s-c));
}

double Tetrakaidecahedra::quadArea(int point1, int point2, int point3, int point4)
{
	double area = trangleArea(point1, point2, point3);
	area += trangleArea(point1, point3, point4);
	return area;
}

double Tetrakaidecahedra::hexagonArea(int point1, int point2, int point3, int point4, int point5, int point6)
{
	double area = trangleArea(point1, point2, point3);
	area += trangleArea(point1, point3, point4);
	area += trangleArea(point1, point4, point5);
	area += trangleArea(point1, point5, point6);
	return area;
}

double Tetrakaidecahedra::getEdgeOfOneUniformTetra()
{
	// For one uniform tetrakaidecahedra only
	return 36.0*distance(0, 1);
}

double Tetrakaidecahedra::getSurfaceAreaOfOneUniformTetra()
{
	// For one uniform tetrakaidecahedra only
	return 6.0*quadArea(0,1,2,3)+8.0*hexagonArea(0,14,13,16,19,1);
}

bool Tetrakaidecahedra::atSamePlane(int point1, int point2, int point3, int point4)
{
	double *pt1=vtx+3*point1;
	double *pt2=vtx+3*point2;
	double *pt3=vtx+3*point3;
	double *pt4=vtx+3*point4;
	double nx, ny, nz, nk, dif;
	nx = (*(pt2+1)-*(pt1+1))*(*(pt3+2)-*(pt1+2))-(*(pt2+2)-*(pt1+2))*(*(pt3+1)-*(pt1+1));
	ny = (*(pt2+2)-*(pt1+2))*(*pt3-*pt1)-(*pt2-*pt1)*(*(pt3+2)-*(pt1+2));
	nz = (*pt2-*pt1)*(*(pt3+1)-*(pt1+1))-(*(pt2+1)-*(pt1+1))*(*pt3-*pt1);
	nk = nx*(*pt1)+ny*(*(pt1+1))+nz*(*(pt1+2));
	dif = nx*(*pt4)+ny*(*(pt4+1))+nz*(*(pt4+2))-nk;
	pt1=NULL;
	pt2=NULL;
	pt3=NULL;
	pt4=NULL;
	if(dif>dbacurate || dif<-dbacurate)
		return false;
	else
		return true;
}

double Tetrakaidecahedra::getSurfaceAreaOfOneRandomTetra()
{
	// For one uniform tetrakaidecahedra only
	double surfarea=0;
	surfarea += quadArea(3,0,1,2);
	surfarea += quadArea(7,4,5,6);
	surfarea += quadArea(8,9,10,11);
	surfarea += quadArea(12,13,14,15);
	surfarea += quadArea(19,16,17,18);
	surfarea += quadArea(23,20,21,22);
	surfarea += hexagonArea(1,0,14,13,16,19);
	surfarea += hexagonArea(7,17,16,13,12,4);
	surfarea += hexagonArea(10,9,18,17,7,6);
	surfarea += hexagonArea(8,2,1,19,18,9);
	surfarea += hexagonArea(21,20,15,14,0,3);
	surfarea += hexagonArea(23,5,4,12,15,20);
	surfarea += hexagonArea(11,10,6,5,23,22);
	surfarea += hexagonArea(11,22,21,3,2,8);
	return surfarea;
}

double Tetrakaidecahedra::getSurfaceAreaOfAllRandomTetra()
{
	double surfarea=0;
	for(int i=0; i<ngx; i++)
		for(int j=0; j<ngy; j++)
			for(int k=0; k<ngz; k++){
				if((i%2==0 && j%2==0 && k%2==0)||(i%2==1 && j%2==1 && k%2==1)){
					getOneTetrakaideVtx(i,j,k);
					surfarea += getSurfaceAreaOfOneRandomTetra();
				}
			}
	return 0.5*surfarea;
}

double Tetrakaidecahedra::getPrimaryEdgeOfAllRandomTetra()
{
	double edgelength = 0.0;
	for(int i=0; i<ngx; i++)
		for(int j=0; j<ngy; j++)
			for(int k=0; k<ngz; k++){
				if((i%2==0 && j%2==0 && k%2==0)||(i%2==1 && j%2==1 && k%2==1)){
					getOneTetrakaideVtx(i,j,k);
					edgelength += getNaturalEdgeLengthOfOneTetra()/3.0;
				}
			}
	return edgelength;
}

double Tetrakaidecahedra::getSecondaryEdgeOfAllRandomTetra()
{
	double edgelength = 0.0;
	for(int i=0; i<ngx; i++)
		for(int j=0; j<ngy; j++)
			for(int k=0; k<ngz; k++){
				if((i%2==0 && j%2==0 && k%2==0)||(i%2==1 && j%2==1 && k%2==1)){
					getOneTetrakaideVtx(i,j,k);
					edgelength += getAnomalousEdgeLengthOfOneTetra()/2.0;
				}
			}
	return edgelength;
}

double Tetrakaidecahedra::getEdgeOfAllRandomTetra()
{
	double edgelength = 0.0;
	for(int i=0; i<ngx; i++)
		for(int j=0; j<ngy; j++)
			for(int k=0; k<ngz; k++){
				if((i%2==0 && j%2==0 && k%2==0)||(i%2==1 && j%2==1 && k%2==1)){
					getOneTetrakaideVtx(i,j,k);
					edgelength += getNaturalEdgeLengthOfOneTetra()/3.0;
					edgelength += getAnomalousEdgeLengthOfOneTetra()/2.0;
				}
			}
	return edgelength;
}

void Tetrakaidecahedra::getGrainCentrePos(int & xpos, int & ypos, int & zpos)
{
	if(xpos%2==1 && ypos%2==1 && zpos%2==1){ 
		xgc = (xpos/2+1)*gs;  
        ygc = (ypos/2+1)*gs; 
        zgc = (zpos/2+1)*gs; 
	}else if(xpos%2==0 && ypos%2==0 && zpos%2==0){
        xgc = (xpos/2+0.5)*gs; 
        ygc = (ypos/2+0.5)*gs; 
        zgc = (zpos/2+0.5)*gs; 
	}
}

void Tetrakaidecahedra::zoomAllRandomTetra(int caseNumber)
{
	double volume=0;
	double manip;
	for(int i=0; i<ngx; i++)
		for(int j=0; j<ngy; j++)
			for(int k=0; k<ngz; k++)
				if((i%2==0 && j%2==0 && k%2==0)||(i%2==1 && j%2==1 && k%2==1)){
					getOneTetrakaideVtx(i,j,k);
					if(caseNumber == 1)
						manip = rdm.getValue();
					else if(caseNumber == 2)
						manip = sqrt(rdm.getValue());
					else if(caseNumber == 3)
					    manip = sqrt(sqrt(rdm.getValue()));
					else if(caseNumber == 4){
						manip = rdm.getValue();
						manip = 1.0/exp(2.0*(manip-1)*(manip-1));}
					else{
						cout<<"the case has not beed defined, aborted\n";
						exit(1);
					}
					zoomOneGrainVtxTowardCenter(i,j,k,manip);
			}
}

void Tetrakaidecahedra::zoomOneGrainVtxTowardCenter(int & xpos, 
						int & ypos, int & zpos, double zoom)
{
	double volume=0.0;
	getGrainCentrePos(xpos, ypos, zpos);
	for(int i=0; i<24; i++){
		vtx[i*3]= zoom * vtx[i*3] + (1-zoom)*xgc;
		vtx[i*3+1] = zoom * vtx[i*3+1] + (1-zoom)*ygc;
		vtx[i*3+2] = zoom * vtx[i*3+2] + (1-zoom)*zgc;
	}
	setOneTetrakaideVtx(xpos, ypos, zpos);
}

double Tetrakaidecahedra::getEdgeOfOneRandomTetra()
{
	// For one uniform tetrakaidecahedra only
	double edgelength = 0.0;
	edgelength += getNaturalEdgeLengthOfOneTetra();
	edgelength += getAnomalousEdgeLengthOfOneTetra();
	return edgelength;
}

double Tetrakaidecahedra::getPrimaryEdgeOfOneRandomTetra()
{
	return getNaturalEdgeLengthOfOneTetra();
}

double Tetrakaidecahedra::getSecondaryEdgeOfOneRandomTetra()
{
	return getAnomalousEdgeLengthOfOneTetra();
}

void Tetrakaidecahedra::sortVolumeCategary()
{
	double *thisvo = new double[totalGrainNumber()];
	int kkk = 0;
	for(int i=0; i<ngx; i++)
		for(int j=0; j<ngy; j++)
			for(int k=0; k<ngz; k++)
				if((i%2==0 && j%2==0 && k%2==0)||(i%2==1 && j%2==1 && k%2==1)){
					getOneTetrakaideVtx(i,j,k);
					getGrainCentrePos(i,j,k);
					thisvo[kkk] = getVolumeOfOneRandomTetra();
					kkk++;
				}
	double minv = thisvo[0];
	double maxv = thisvo[0];
	
	for(int i=1; i<kkk; i++){
		if(thisvo[i]>maxv)
			maxv = thisvo[i];
		if(thisvo[i]<minv)
			minv = thisvo[i];
	}
	minv = 0;
	maxv = 120;
	cout<<totalGrainNumber()<<" "<<kkk<<" "<<maxv<<" "<<minv<<endl; 
	double spanv = (maxv-minv)/20.0;
	int numg[20];
	for(int i=0; i<20; i++)
		numg[i] = 0;
	for(int i=0; i<kkk; i++){
		for(int j=0; j<20; j++)
		 if(thisvo[i]>minv+j*spanv && (thisvo[i]<minv+(j+1)*spanv))
			 numg[j]++;
	}
	for(int i=0; i<20; i++)
		volfile<<minv+0.5*(2*i+1)*spanv<<" "<<numg[i]<<endl;
}


double Tetrakaidecahedra::getVolumeOfAllRandomTetra()
{
	double volume=0;
	for(int i=0; i<ngx; i++)
		for(int j=0; j<ngy; j++)
			for(int k=0; k<ngz; k++)
				if((i%2==0 && j%2==0 && k%2==0)||(i%2==1 && j%2==1 && k%2==1)){
					getOneTetrakaideVtx(i,j,k);
					getGrainCentrePos(i,j,k);
					volume += getVolumeOfOneRandomTetra();
				}
	return volume;
}

void Tetrakaidecahedra::deformGrainCenter()
{
	double temx, temy, temz;
	temx = sij[0] * xgc + sij[1] * ygc + sij[2] * zgc;
	temy = sij[3] * xgc + sij[4] * ygc + sij[5] * zgc;
	temz = sij[6] * xgc + sij[7] * ygc + sij[8] * zgc;
	xgc = temx;
	ygc = temy;
	zgc = temz;
}

double Tetrakaidecahedra::getVolumeOfOneRandomTetra()
{
	double volume=0.0;
	deformGrainCenter();
	
	volume += getVolumeTriangularPyramid(1,0,3);
	volume += getVolumeTriangularPyramid(1,3,2);

	volume += getVolumeTriangularPyramid(5,4,7);
	volume += getVolumeTriangularPyramid(5,7,6);

	volume += getVolumeTriangularPyramid(8,10,9);
	volume += getVolumeTriangularPyramid(8,11,10);

	volume += getVolumeTriangularPyramid(12,14,13);
	volume += getVolumeTriangularPyramid(12,15,14);

	volume += getVolumeTriangularPyramid(19,17,16);
	volume += getVolumeTriangularPyramid(19,18,17);

	volume += getVolumeTriangularPyramid(21,20,23);
	volume += getVolumeTriangularPyramid(21,23,22);

	volume += getVolumeTriangularPyramid(1,19,16);
	volume += getVolumeTriangularPyramid(1,16,13);
	volume += getVolumeTriangularPyramid(1,13,14);
	volume += getVolumeTriangularPyramid(1,14,0);

	volume += getVolumeTriangularPyramid(7,4,12);
	volume += getVolumeTriangularPyramid(7,12,13);
	volume += getVolumeTriangularPyramid(7,13,16);
	volume += getVolumeTriangularPyramid(7,16,17);

	volume += getVolumeTriangularPyramid(10,6,7);
	volume += getVolumeTriangularPyramid(10,7,17);
	volume += getVolumeTriangularPyramid(10,17,18);
	volume += getVolumeTriangularPyramid(10,18,9);

	volume += getVolumeTriangularPyramid(8,9,18);
	volume += getVolumeTriangularPyramid(8,18,19);
	volume += getVolumeTriangularPyramid(8,19,1);
	volume += getVolumeTriangularPyramid(8,1,2);

	volume += getVolumeTriangularPyramid(21,3,0);
	volume += getVolumeTriangularPyramid(21,0,14);
	volume += getVolumeTriangularPyramid(21,14,15);
	volume += getVolumeTriangularPyramid(21,15,20);

	volume += getVolumeTriangularPyramid(23,20,15);
	volume += getVolumeTriangularPyramid(23,15,12);
	volume += getVolumeTriangularPyramid(23,12,4);
	volume += getVolumeTriangularPyramid(23,4,5);

	volume += getVolumeTriangularPyramid(11,22,23);
	volume += getVolumeTriangularPyramid(11,23,5);
	volume += getVolumeTriangularPyramid(11,5,6);
	volume += getVolumeTriangularPyramid(11,6,10);

	volume += getVolumeTriangularPyramid(11,8,2);
	volume += getVolumeTriangularPyramid(11,2,3);
	volume += getVolumeTriangularPyramid(11,3,21);
	volume += getVolumeTriangularPyramid(11,21,22);
	return volume;
}

double Tetrakaidecahedra::getVolumeTriangularPyramid(int point1, int point2, int point3)
{
	double a,b,c,d,s,height, area, volume;
	double *pta=vtx+3*point1;
	double *ptb=vtx+3*point2;
	double *ptc=vtx+3*point3;
	a = (*(ptb+1)-*(pta+1))*(*(ptc+2)-*(pta+2))
		-(*(ptb+2)-*(pta+2))*(*(ptc+1)-*(pta+1));
	b = (*(ptb+2)-*(pta+2))*(*(ptc)-*(pta))
		-(*(ptb)-*(pta))*(*(ptc+2)-*(pta+2));
	c = (*(ptb)-*(pta))*(*(ptc+1)-*(pta+1))
		-(*(ptb+1)-*(pta+1))*(*(ptc)-*(pta));
	d = - a*(*pta)-b*(*(pta+1))-c*(*(pta+2));
	s = sqrt(a*a+b*b+c*c);
	pta=NULL;
	ptb=NULL;
	ptc=NULL;
	height = (a*xgc+b*ygc+c*zgc+d)/s;
	if(height<0){
		printf("found negative volume\n"); exit(0);
	}
	area = trangleArea(point1, point2, point3);
	volume = height * area / 3.0;	
	return volume;
}

void Tetrakaidecahedra::ramdomizeVtx(double randomIntensity)
{
	if(randomIntensity<-1 || randomIntensity>1){
		printf("randomIntensity too large, should be within -1 and 1, aborted\n"); 
		exit(1);
	}
	rdm.setSeed(5000);
	for(int i=0; i<nvt; i++){
		vcx[i] += qgs*(rdm.getValue()-0.5)*randomIntensity;
		vcy[i] += qgs*(rdm.getValue()-0.5)*randomIntensity;
		vcz[i] += qgs*(rdm.getValue()-0.5)*randomIntensity;
	}
}


double Tetrakaidecahedra::getNaturalEdgeLengthOfOneTetra()
{
	double edgelength = 0.0;
	if(!atSamePlane(14,1,3,0)) // 1
		edgelength += distance(0, 1);
	if(!atSamePlane(3,1,8,2)) // 2
		edgelength += distance(1, 2);
	if(!atSamePlane(3,1,2,11)) // 3
		edgelength += distance(2,3);
	if(!atSamePlane(21,0,1,3)) // 4
		edgelength += distance(3,0);
	if(!atSamePlane(23,5,7,4)) // 5
		edgelength += distance(5,4);
	if(!atSamePlane(7,5,11,6)) // 6
		edgelength += distance(5,6);
	if(!atSamePlane(7,5,6,10)) // 7
		edgelength += distance(6,7);
	if(!atSamePlane(7,12,4,5)) // 8
		edgelength += distance(4,7);
	if(!atSamePlane(8,18,9,10)) // 9
		edgelength += distance(8,9);
	if(!atSamePlane(8,18,9,10)) // 10
		edgelength += distance(9,10);
	if(!atSamePlane(8,10,6,11)) // 11
		edgelength += distance(10,11);
	if(!atSamePlane(8,10,11,2)) // 12
		edgelength += distance(8,11);
	if(!atSamePlane(12,7,13,14)) // 13
		edgelength += distance(12,13);
	if(!atSamePlane(12,13,1,14)) // 14
		edgelength += distance(13,14);
	if(!atSamePlane(12,14,21,15)) // 15
		edgelength += distance(14,15);
	if(!atSamePlane(12,14,15,23)) // 16
		edgelength += distance(12,15);
	if(!atSamePlane(16,7,17,19)) // 17
		edgelength += distance(16,17);
	if(!atSamePlane(19,17,10,18)) // 18
		edgelength += distance(17,18);
	if(!atSamePlane(19,17,18,8)) // 19
		edgelength += distance(18,19);
	if(!atSamePlane(1,16,17,19)) // 20
		edgelength += distance(16,19);
	if(!atSamePlane(15,21,23,20)) // 21
		edgelength += distance(20,21);
	if(!atSamePlane(23,21,11,22)) // 22
		edgelength += distance(21,22);
	if(!atSamePlane(23,21,22,11)) // 23
		edgelength += distance(22,23);
	if(!atSamePlane(15,20,21,23)) // 24
		edgelength += distance(20,23);
	if(!atSamePlane(21,14,1,0)) // 25
		edgelength += distance(0,14);
	if(!atSamePlane(13,7,16,1)) // 26
		edgelength += distance(13,16);
	if(!atSamePlane(1,16,19,8)) // 27
		edgelength += distance(1,19);
	if(!atSamePlane(12,23,4,7)) // 28
		edgelength += distance(4,12);
	if(!atSamePlane(7,10,17,16)) // 29
		edgelength += distance(7,17);
	if(!atSamePlane(6,11,10,7)) // 30
		edgelength += distance(6,10);
	if(!atSamePlane(9,8,18,10)) // 31
		edgelength += distance(9,18);
	if(!atSamePlane(8,11,2,1)) // 32
		edgelength += distance(8,2);
	if(!atSamePlane(21,20,23,15)) // 33
		edgelength += distance(20,15);
	if(!atSamePlane(23,11,5,4)) // 34
		edgelength += distance(23,5);
	if(!atSamePlane(21,11,23,22)) // 35
		edgelength += distance(11,22);
	if(!atSamePlane(21,0,3,11)) // 36
		edgelength += distance(3,21);
	return edgelength;
}

double Tetrakaidecahedra::getAnomalousEdgeLengthOfOneTetra()
{
	double edgelength = 0.0;
	if(!atSamePlane(0,1,2,3)) // 37
		edgelength += distance(1,3);
	if(!atSamePlane(16,17,18,19)) // 37 -- from wrong notice
		edgelength += distance(17,19);
	if(!atSamePlane(4,5,6,7)) // 38
		edgelength += distance(5,7);
	if(!atSamePlane(20,21,22,23)) // 39
		edgelength += distance(21,23);
	if(!atSamePlane(8,9,10,11)) // 40
		edgelength += distance(8,10);
	if(!atSamePlane(12,13,14,15)) // 41
		edgelength += distance(12,14);
	if(!atSamePlane(1,0,14,13)) // 42
		edgelength += distance(1,14);
	if(!atSamePlane(1,14,13,16)) // 43
		edgelength += distance(1,13);
	if(!atSamePlane(1,13,16,19)) // 44
		edgelength += distance(1,16);
	if(!atSamePlane(23,22,11,5)) // 45
		edgelength += distance(23,11);
	if(!atSamePlane(23,11,6,5)) // 46
		edgelength += distance(11,5);
	if(!atSamePlane(5,11,10,6)) // 47
		edgelength += distance(11,6);
	if(!atSamePlane(8,2,1,19)) // 48
		edgelength += distance(8,1);
	if(!atSamePlane(8,1,19,18)) // 49
		edgelength += distance(8,19);
	if(!atSamePlane(8,19,18,9)) // 50
		edgelength += distance(8,18);
	if(!atSamePlane(23,5,4,12)) // 51
		edgelength += distance(23,4);
	if(!atSamePlane(23,4,12,15)) // 52
		edgelength += distance(23,12);
	if(!atSamePlane(23,12,15,20)) // 53
		edgelength += distance(23,15);
	if(!atSamePlane(21,20,15,14)) // 54
		edgelength += distance(21,15);
	if(!atSamePlane(21,15,14,0)) // 55
		edgelength += distance(21,14);
	if(!atSamePlane(21,14,0,3)) // 56
		edgelength += distance(21,0);
	if(!atSamePlane(10,6,7,17)) // 57
		edgelength += distance(19,7);
	if(!atSamePlane(10,7,17,18)) // 58
		edgelength += distance(10,17);
	if(!atSamePlane(10,17,18,9)) // 59
		edgelength += distance(10,18);
	if(!atSamePlane(7,17,16,13)) // 60
		edgelength += distance(7,16);
	if(!atSamePlane(7,16,13,12)) // 61
		edgelength += distance(7,13);
	if(!atSamePlane(7,13,12,4)) // 62
		edgelength += distance(7,12);
	if(!atSamePlane(11,3,2,8)) // 63
		edgelength += distance(11,2);
	if(!atSamePlane(11,21,3,2)) // 64
		edgelength += distance(11,3);
	if(!atSamePlane(11,22,21,3)) // 65
		edgelength += distance(11,21);
	return edgelength;
}

#endif // TETRAKAIDECAHEDRA_CPP